import itertools
from collections import deque
from time import time
from weakref import WeakSet #TODO: for debugging
from logzero import logger as log


class Cache:

    instances = WeakSet() #TODO: for debugging

    def __init__(self, size=100):
        self.size = size
        self.times  = deque(maxlen=size)
        self.values = deque(maxlen=size)
        self.instances.add(self) #TODO: for debugging

    def callback_append(self, value=None, **kwargs):
        self.append(value)

    def append(self, value):
        self.times.append(time())
        self.values.append(value)

    def __iter__(self):
        return iter(self.data)

    @property
    def data(self):
        return self.times, self.values

#    @property
#    def latest(self):
#        try:
#            return self.values[-1]
#        except IndexError:
#            return None

    def __repr__(self):
        n = len(self.times)
        return f"Cache: {n} / {self.size}"

    def set_size(self, size):
        self.size = size
        self.times  = deque(self.times,  maxlen=size)
        self.values = deque(self.values, maxlen=size)

    def get_view(self, size=None):
        size = size or self.size
        return CacheView(self, size=size)



class CacheView:

    instances = WeakSet() #TODO: for debugging

    def __init__(self, cache, size=100):
        self.cache = cache
        self.size = size
        self.instances.add(self) #TODO: for debugging

    def __iter__(self):
        return iter(self.data)

    @property
    def data(self):
        return tuple(get_last(i, self.size) for i in self.cache.data)

    def __repr__(self):
        return f"CacheView ({self.size}) on {self.cache}"

    def set_size(self, size):
        self.size = size
        if size > self.cache.size:
            log.debug("Adjust cache size to view size...")
            self.cache.set_size(size)



def get_last(d, n):
    stop = len(d)
    start = stop - n
    start = max(start, 0)
    return list(itertools.islice(d, start, stop))



#TODO: for debugging

def print_caches():
    cs = get_caches()
    vs = get_views()
    print(f"Caches: {cs}\nViews:  {vs}")

def get_caches():
    return set(Cache.instances)

def get_views():
    return set(CacheView.instances)



#TODO: remove caches without any views?



