from random import choice
from logzero import logger as log

from bokeh.layouts import column
from bokeh.models import Spacer, TextInput
from bokeh.plotting import curdoc

from .actor import Actor
from .buki import Column, Row
from .widgets import Files
from .utils import bind


class Director:

    def __init__(self):
        self.doc = doc = curdoc()
        doc.title = choice("👺👹") + " kabuki"

        self.ti_add_pvs = ti_add_pvs = TextInput(value="", title="Add PV:")
        bind(ti_add_pvs, self.do_add_pvs)

        files = Files("config")
        files.on_click_save(self.get_pvs_grid)
        files.on_click_load(self.add_pvs_grid)

        self.plot_container = plot_container = Column()
        root_container = column(ti_add_pvs, files.layout, Spacer(height=15), plot_container.layout)
        doc.add_root(root_container)

        self.updates = []
        self.doc.add_periodic_callback(self.periodic_update, 1000)


    def periodic_update(self):
        for func in self.updates:
            func()
#            self.doc.add_next_tick_callback(func) #TODO: would this be an improvement?


    def do_add_pvs(self, new):
        if not new: return # do not trigger on the following value reset
        self.ti_add_pvs.value = ""
        self.add_pvs(new)
#        self.doc.add_next_tick_callback(lambda: self.add_pvs(new)) #TODO: would this be an improvement?


    def add_pvs(self, new):
        new = new.upper() # PV names are all upper case
        grid = break_apart(new)
        log.debug(normalized_string(grid))
        self.add_pvs_grid(grid)


    def add_pvs_grid(self, pvnames_grid):
        for pvnames in reversed(pvnames_grid):
            pvs_row = self.make_pvs_row(pvnames)
            if not pvs_row.children:
                continue # do not insert empty rows
            self.plot_container.prepend(pvs_row)


    def make_pvs_row(self, pvnames):
        container = Row()
        for n in pvnames:
            log.info(f"Add PV: {n}")
            try:
                a = Actor(n)
            except Exception as e:
                log.exception(e)
            else:
                self.add_actor(a, container)
        return container


    def add_actor(self, a, container):
        log.info(f"Add: {a}")
        log.debug("Connect close button")
        a.frm.on_click_close(lambda: self.remove_actor(a, container))
        log.debug("Add plot")
        container.append(a.frm)
        log.debug("Add update")
        self.updates.append(a.update)


    def remove_actor(self, a, container):
        log.info(f"Remove: {a}")
        log.debug("Remove update")
        self.updates.remove(a.update)
        log.debug("Remove plot")
        container.remove(a.frm)
        log.debug("Delete")
        a.delete()


    def get_pvs_grid(self):
        res = [[col.plt.name for col in row] for row in self.plot_container]
        log.debug(normalized_string(res))
        return res



MAPPING = str.maketrans({
    "\"": " ",
    "\'": " ",
    ",": " ",
    ";": "\n"
})

def break_apart(s):
    s = s.translate(MAPPING)
    lines = s.split("\n")
    res = [i.split() for i in lines]
    return res

def normalized_string(grid):
    return "; ".join(", ".join(names) for names in grid)



