from bokeh.models import Button, Div, Spacer

from .buki import Column, Row


CROSS = "🗙"
GEAR = "⚙"


class Frame(Column):

    def __init__(self, plt, cfg):
        self.plt = plt
        self.cfg = cfg

#        self.set = plt.set # make frame act like contained plot
        cfg.visible = False

        name = plt.name

        n_btns = 2
        btn_width = 35
        lbl_width = plt.width - n_btns * btn_width

        self.btn_close = btn_close = Button(label=CROSS, button_type="default", width=btn_width)
        self.btn_cfg   = btn_cfg   = Button(label=GEAR,  button_type="default", width=btn_width)

        lbl = Div(text=name, align="center", style={"font-size": "150%"}, width=lbl_width)
        header = Row(btn_close, btn_cfg, lbl)

        self.btn_cfg.on_click(self.do_click_cfg)

        self.dummy = dummy = Div(width=0, height=0)

        # wrap fig and cfg such that their container can be hidden during the visibility switch
        # wrap with another container to maintain size
        # combined gives a relatively smooth (albeit slow) transition between the two states
        self.inner = inner = Row(plt, cfg)
        size_maintainer = Row(inner, min_width=plt.width, min_height=plt.height) #TODO: why min_* ?

        super().__init__(header, size_maintainer, Spacer(height=35), dummy, name=name)


    def on_click_close(self, handler):
        def wrapper():
            if self.btn_close.disabled: return # workaround for delay between disable in python and in browser
            self.btn_close.disabled = True     # avoid running handler multiple times due to too fast clicks
            handler()
        self.btn_close.on_click(wrapper)

    def do_click_cfg(self):
        toggle_button_type(self.btn_cfg, "default", "warning")
        toggle_button_state(self.btn_close) # do not close plot from cfg dialog
        self.inner.visible = False
        switch_visibility(self.plt, self.cfg)
        self.inner.visible = True
        toggle_visibility(self.dummy) # workaround for overlapping widgets



def toggle_button_state(btn):
    btn.disabled = not btn.disabled

def toggle_button_type(btn, type1, type2):
    btn.button_type = type2 if btn.button_type == type1 else type1

def switch_visibility(a, b):
    a.visible, b.visible = b.visible, a.visible

def toggle_visibility(a):
    a.visible = not a.visible



