from .cfgdlgs import ConfigDialog0D, ConfigDialog1D, ConfigDialog2D
from .plots import Plot0D, Plot1D, Plot2D
from .sources import Camera, Source


src_cache = {}

def get_src(pvname, cls):
    try:
        src = src_cache[pvname]
        assert isinstance(src, cls)
    except KeyError:
        src = src_cache[pvname] = cls(pvname)
    return src


def decide_src_plt_cfg(pvname):
    Src = decide_src_type(pvname)
    src = get_src(pvname, Src)

    if not src.is_connected:
        raise RuntimeError(f"{pvname} is not connected, cannot create plot")

    Plt, Cfg = decide_plt_cfg_type(src)
    plt = Plt(name=pvname)
    cfg = Cfg()

    return src, plt, cfg


def decide_src_type(pvname):
    if pvname.endswith(":FPICTURE"):
        return Camera
    return Source


def decide_plt_cfg_type(src):
    if isinstance(src, Camera):
        return Plot2D, ConfigDialog2D

    nelm = src.nelm
    if nelm == 1:
        return Plot0D, ConfigDialog0D
    elif nelm > 1:
        return Plot1D, ConfigDialog1D

    raise RuntimeError(f"Cannot decide plot type for {src}: {src.value}")



