import numpy as np

from bokeh.layouts import row
from bokeh.models import ColumnDataSource, HoverTool
from bokeh.plotting import figure

from ..buki import Object
from ..utils import make_tooltips


TOOLTIPS_TIMESERIES = make_tooltips(
    x = "$x",
    y = "$y",
)

TOOLTIPS_HISTO = make_tooltips(
    x = "$x",
    y = "$y",
    bin = "@left",
    entries = "@top"
)


class Plot0D(Object):

    def __init__(self, name=None):
        self.timeseries = timeseries = TimeSeries()
        self.histo = histo = Histo()

        #TODO: if not set, width and height are None
        width = timeseries.fig.width + histo.fig.width
        height = max(timeseries.fig.height, histo.fig.height)

        fig = row(timeseries.fig, histo.fig, width=width, height=height, name=name)
        super().__init__(fig)


    def set(self, times, values):
        self.timeseries.set(times, values)
        self.histo.set(values)



class TimeSeries:

    def __init__(self):
        data = {
            "x": [],
            "y": []
        }

        self.source = source = ColumnDataSource(data=data)

        self.fig = fig = figure()
        fig.toolbar.logo = None
        fig.step(x="x", y="y", source=source, mode="after")
        fig.circle(x="x", y="y", source=source)

        hover = HoverTool(tooltips=TOOLTIPS_TIMESERIES, mode="vline")
        fig.add_tools(hover)
        fig.toolbar.active_inspect = None


    def set(self, times, values):
        data = {
            "x": times,
            "y": values
        }

        self.source.data.update(data)



class Histo:

    def __init__(self):
        data = {
            "left": [],
            "right": [],
            "top": []
        }

        self.source = source = ColumnDataSource(data)

        self.fig = fig = figure()
        fig.toolbar.logo = None
        fig.quad(left="left", right="right", top="top", bottom=0, source=source)

        hover = HoverTool(tooltips=TOOLTIPS_HISTO, mode="vline")
        fig.add_tools(hover)
        fig.toolbar.active_inspect = None


    def set(self, values):
        data = hist(values)
        self.source.data.update(data)



def hist(values, bins="auto"):
    counts, edges = np.histogram(values, bins=bins)
    left  = edges[:-1]
    right = edges[1:]
    return dict(left=left, right=right, top=counts)



