import numpy as np

from bokeh.models import ColumnDataSource, ColorBar, LinearColorMapper
from bokeh.plotting import figure

from ..colormaps import cmaps
from ..buki import Object
from ..utils import make_tooltips


TOOLTIPS = make_tooltips(
    x = "$x",
    y = "$y",
    z = "@image",
)


class Plot2D(Object):

    def __init__(self, name=None):
        data = {
            "image": [np.zeros((1, 1))]
        }

        self.source = source = ColumnDataSource(data=data)

        self.binning = 1

        self.cmap = cmap = LinearColorMapper()
        self.cmap_palette = "Viridis"
        self.cmap_reverse = False
        self._update_cmap()
        cbar = ColorBar(color_mapper=cmap)

        fig = figure(name=name, match_aspect=True, tooltips=TOOLTIPS)
        fig.toolbar.logo = None
        fig.toolbar.active_inspect = None
        fig.image(source=source, x=0, y=0, dw=1, dh=1, color_mapper=cmap)

        fig.x_range.range_padding = 0
        fig.y_range.range_padding = 0

        fig.add_layout(cbar, "right")
        super().__init__(fig)


    def set(self, _times, values):
        image = values[-1]
        image = rebin(image, self.binning)
        self.source.data.update(image=[image])


    def change_cmap_palette(self, name):
        self.cmap_palette = name
        self._update_cmap()

    def change_cmap_reverse(self, state):
        self.cmap_reverse = state
        self._update_cmap()

    def _update_cmap(self):
        pal = cmaps[self.cmap_palette]
        if self.cmap_reverse:
            pal = pal[::-1]
        self.cmap.palette = pal

    def change_vmin(self, value):
        self.cmap.low = value

    def change_vmax(self, value):
        self.cmap.high = value

    def change_binning(self, value):
        self.binning = value



def rebin(img, binning):
    if binning == 1:
        return img

    old_shape = np.array(img.shape)
    new_ny, new_nx = new_shape = old_shape // binning
    cut_shape = new_shape * binning

    left_ny, left_nx = offsets = (old_shape - cut_shape) // 2
    right_ny, right_nx = cut_shape + offsets

    meaning_shape = (
        new_ny, binning,
        new_nx, binning
    )

    img = img[left_ny:right_ny, left_nx:right_nx]
    img.shape = meaning_shape
    img = img.mean(axis=(1, 3))
    return img



#TODO:
#- projections



