from functools import wraps

from .cachedsource import CachedSource
from .pvcollection import PVCollection


class Camera(CachedSource, PVCollection):
    """
    Expects: pvname = "NAME:FPICTURE"
    Also creates NAME:WIDTH and NAME:HEIGHT
    """

    #TODO: could also accept base as argument, and also append ":FPICTURE"

    def __init__(self, pvname):
        self.pvname = pvname
        assert pvname.endswith(":FPICTURE")
        base = pvname.rsplit(":", 1)[0]
        PVCollection.__init__(
            self,
            base,
            image = pvname,
            width = base + ":WIDTH",
            height = base + ":HEIGHT"
        )
        CachedSource.__init__(self, size=1)
        self.pvs.image.auto_monitor = True # auto-monitor is off by default for large arrays, callbacks are not triggered without auto-monitor

    def get(self):
        data = self.pvs.image.get()
        data.shape = self.get_shape()
        return data

    def get_shape(self):
        width = self.pvs.width.get()
        height = self.pvs.height.get()
        width = int(round(width))
        height = int(round(height))
        shape = (height, width)
        return shape

    def add_callback(self, callback, **kwargs):
        @wraps(callback)
        def wrapper(*args, value=None, **kwargs):
            value.shape = self.get_shape()
            return callback(*args, value=value, **kwargs)
        return self.pvs.image.add_callback(callback=wrapper, **kwargs)

    def remove_callback(self, *args, **kwargs):
        return self.pvs.image.remove_callback(*args, **kwargs)

    def clear_callbacks(self):
        self.pvs.image.clear_callbacks()



