import json
from pathlib import Path
from logzero import logger as log

from bokeh.models import AutocompleteInput, Button, Div

from ..buki import Column, Row
from ..utils import adjust_margin


BTN_WIDTH = 100


class Files(Column):

    def __init__(self, folder, title="Files", name=None, sizing_mode="stretch_width"):
        self.folder = folder = Path(folder)
        files = [str(i.relative_to(folder)) for i in folder.rglob("*")]

        lbl =  Div(text=title)
        adjust_margin(lbl, bottom=-10)

        self.ai_fname = ai_fname = AutocompleteInput(value="", completions=files, restrict=False, min_characters=1, sizing_mode=sizing_mode)

        self.btn_save = btn_save = Button(label="⮫🖫 save", width=BTN_WIDTH)
        self.btn_load = btn_load = Button(label="🖫⮯ load",  width=BTN_WIDTH)

        inner = Row(ai_fname, btn_save, btn_load)
        super().__init__(lbl, inner, sizing_mode=sizing_mode)


    def on_click_save(self, handler):
        def wrapper(_event):
            fn = self.get_fname()
            if not fn:
                log.warning("Clicked save with empty filename")
                return

            self.add_completion(fn)

            fn = self.folder / fn
            fn.parent.mkdir(parents=True, exist_ok=True)
            log.debug(f"Save to {fn}")

            data = handler()
            json_save(data, fn)
        self.btn_save.on_click(wrapper)


    def on_click_load(self, handler):
        def wrapper(_event):
            fn = self.get_fname()
            if not fn:
                log.warning("Clicked load with empty filename")
                return

            fn = self.folder / fn
            if not fn.exists():
                log.error(f"Cannot load: {fn} does not exist")
                return
            log.debug(f"Load from {fn}")

            data = json_load(fn)
            handler(data)
        self.btn_load.on_click(wrapper)


    def get_fname(self):
        return self.ai_fname.value.strip()


    def add_completion(self, fn):
        compl = self.ai_fname.completions
        if fn not in compl:
            compl.append(fn)



def json_save(what, filename, *args, indent=4, sort_keys=True, **kwargs):
    with open(filename, "w") as f:
        json.dump(what, f, *args, indent=indent, sort_keys=sort_keys, **kwargs)

def json_load(filename, *args, **kwargs):
    with open(filename, "r") as f:
        return json.load(f, *args, **kwargs)



